package ch.fritteli.labyrinth.generator;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.renderer.html.HTMLRenderer;
import ch.fritteli.labyrinth.generator.renderer.htmlfile.HTMLFileRenderer;
import ch.fritteli.labyrinth.generator.renderer.pdffile.PDFFileRenderer;
import ch.fritteli.labyrinth.generator.renderer.text.TextRenderer;
import ch.fritteli.labyrinth.generator.renderer.textfile.TextFileRenderer;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

import java.nio.file.Path;
import java.nio.file.Paths;

@Slf4j
public class Main {
    public static void main(@NonNull final String[] args) {
        final int width = 20;
        final int height = 30;
        final Labyrinth labyrinth = new Labyrinth(width, height/*, 0*/);
        final TextRenderer textRenderer = TextRenderer.newInstance();
        final HTMLRenderer htmlRenderer = HTMLRenderer.newInstance();
        final Path userHome = Paths.get(System.getProperty("user.home"));
        final String baseFilename = getBaseFilename(labyrinth);
        final TextFileRenderer textFileRenderer = TextFileRenderer.newInstance()
                .setTargetLabyrinthFile(userHome.resolve(baseFilename + ".txt"))
                .setTargetSolutionFile(userHome.resolve(baseFilename + "-solution.txt"));
        final HTMLFileRenderer htmlFileRenderer = HTMLFileRenderer.newInstance()
                .setTargetFile(userHome.resolve(baseFilename + ".html"));
        final PDFFileRenderer pdfFileRenderer = PDFFileRenderer.newInstance()
                .setTargetFile(userHome.resolve(baseFilename + ".pdf"));

        log.info("Labyrinth-ID: {}", labyrinth.getRandomSeed());
        // Render Labyrinth to stdout
        log.info("Text rendering:\n{}", textRenderer.render(labyrinth));
        // Render Labyrinth solution to stdout
        log.info("Text rendering with solution:\n{}", textRenderer.setRenderSolution(true).render(labyrinth));
        // Render HTML to stdout
        log.info("HTML rendering:\n{}", htmlRenderer.render(labyrinth));
        // Render Labyrinth and solution to (separate) files
        log.info("Text rendering to file:\n{}", textFileRenderer.render(labyrinth));
        // Render HTML to file
        log.info("HTML rendering to file:\n{}", htmlFileRenderer.render(labyrinth));
        // Render PDF to file
        log.info("PDF rendering to file:\n{}", pdfFileRenderer.render(labyrinth));
    }

    private static String getBaseFilename(@NonNull final Labyrinth labyrinth) {
        return "labyrinth-" + labyrinth.getWidth() + "x" + labyrinth.getHeight() + "-" + labyrinth.getRandomSeed();
    }
}
